using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using IE = SHDocVw;
using System.Diagnostics;

namespace AddinExpress
{
    namespace AdvancedSearch
    {
        internal struct ModuleDescriptor
        {
            public mshtml.HTMLDocument HTMLDoc;
            public IntPtr TabWnd;
            public IntPtr NativeWnd;
            public IntPtr MainWnd;
            public int ThreadId;
            public HTMLForm Form;
        }

        internal struct SearchDescriptor
        {
            public bool CheckCase;
            public bool CheckHidden;
            public bool CheckTabs;
            public bool CheckWholeWord;
            public bool CheckWindows;
            public string Term;
            public SearchFlags Flags;
            public int SearchModuleIndex;
            public int CurrentModuleIndex;
            public List<ModuleDescriptor> Modules;
            public TreeNode ResultNode;
        }

        [Serializable]
        public class HtmlInfoTag
        {
            public HtmlInfoTag(int line, int position)
            {                
                this.line = line;
                this.position = position;
            }

            private int line;
            private int position;            

            public int Line
            {
                get
                {
                    return this.line;
                }
            }
            public int Position
            {
                get
                {
                    return this.position;
                }
            }            
        }
        [Serializable]
        internal class WindowInfoTag
        {
            public WindowInfoTag(ModuleDescriptor moduleDesc, string term, SearchFlags flags)
            {
                this.term = term;
                this.flags = flags;
                this.moduleDesc = moduleDesc;
            }

            string term;
            SearchFlags flags;
            ModuleDescriptor moduleDesc;

            public IntPtr NativeWindowHandle
            {
                get
                {
                    return moduleDesc.NativeWnd;
                }
            }

            public IntPtr HWND
            {
                get
                {
                    return moduleDesc.MainWnd;
                }
            }

            public int ThreadID
            {
                get
                {
                    return moduleDesc.ThreadId;
                }
            }

            public string Term
            {
                get
                {
                    return term;
                }
            }

            public SearchFlags Flags
            {
                get
                {
                    return flags;
                }
            }
        }

        [Flags]
        [Serializable]
        public enum SearchFlags
        {
            ieMatchPartialWords = 0,
            ieMatchBackwards = 1,
            ieMatchWholeWordsOnly = 2,
            ieMatchCase = 4
        }
        public enum SearchProgress
        {
            None = -1,
            Start = 0,
            Initializing = 10,
            Searching = 50,
            AddingNodes = 80,
            Done = 100
        }
        public static class Const
        {
            // auxiliary
            public const int Ok = 0;
            public const int Found = 0;
            public const int NotFound = -1;
            public const int None = NotFound;
            // Internet Explorer
            public const int IE6 = 6;
            public const int IE7 = 7;
            // window messages
            public const int WM_USER = 0x400;
            public const int WM_SETFOCUS = 0x0007;
            public const int WM_ACTIVATE = 0x0006;
            public const int WM_SHOWWINDOW = 0x0018;
            // custom
            public const int WM_SELECT_TAB = WM_USER + 1000;
            public const int WM_ITEM_CLOSED = WM_USER + 10000;
            public const int WM_SHOW_HTML_PARAMS = WM_USER + 1110;
            public const int WM_SHOW_HTML_FORM = WM_USER + 1111;
            public const int WM_CLOSE_FORMS = WM_USER + 1112;
        }

        namespace VisibleContent
        {
            namespace CaseSensitive
            {
                namespace WholeWord
                {
                    static class AdvancedSearchEngine
                    {
                        internal static List<int> Contains(string element, string term)
                        {
                            string searchCriteria = term;
                            int startIndex = Const.Ok;
                            List<int> indexes = new List<int>();
                            bool standaloneWordAfter = false;
                            bool standaloneWordBefore = false;
                            if (!String.IsNullOrEmpty(element))
                            {
                                while ((startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)) != Const.NotFound)
                                {
                                    standaloneWordAfter = false;
                                    standaloneWordBefore = false;

                                    #region a letter before
                                    if (startIndex != 0)
                                    {
                                        Char letterBefore = element[startIndex - 1];
                                        if (!Char.IsLetterOrDigit(letterBefore))
                                        {
                                            standaloneWordBefore = true;
                                        }
                                        else
                                        {
                                            if (!Char.IsLetterOrDigit(element[startIndex]))
                                            {
                                                standaloneWordBefore = true;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        standaloneWordBefore = true;
                                    }
                                    #endregion

                                    #region a letter after
                                    if (startIndex + term.Length != element.Length)
                                    {
                                        Char letterAfter = element[startIndex + term.Length];
                                        if (!Char.IsLetterOrDigit(letterAfter))
                                        {
                                            standaloneWordAfter = true;
                                        }
                                        else
                                        {
                                            if (!Char.IsLetterOrDigit(element[startIndex + term.Length - 1]))
                                            {
                                                standaloneWordAfter = true;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        standaloneWordAfter = true;
                                    }
                                    #endregion

                                    if (standaloneWordBefore && standaloneWordAfter)
                                    {
                                        indexes.Add(startIndex);
                                    }
                                    startIndex++;
                                }
                            }
                            return indexes;
                        }
                    }
                }

                namespace WordAsPart
                {
                    static class AdvancedSearchEngine
                    {
                        internal static List<int> Contains(string element, string term)
                        {
                            string searchCriteria = term;
                            int startIndex = Const.Ok;
                            List<int> indexes = new List<int>();
                            if (!String.IsNullOrEmpty(element))
                            {
                                while ((startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)) != Const.NotFound)
                                {
                                    indexes.Add(startIndex++);
                                }
                            }
                            return indexes;
                        }
                    }
                }
            }

            namespace CaseInsensetive
            {
                namespace WholeWord
                {
                    static class AdvancedSearchEngine
                    {
                        internal static List<int> Contains(string element, string term)
                        {
                            string searchCriteria = term;
                            int startIndex = Const.Ok;
                            List<int> indexes = new List<int>();
                            bool standaloneWordAfter = false;
                            bool standaloneWordBefore = false;
                            if (!String.IsNullOrEmpty(element))
                            {
                                while ((startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)) != Const.NotFound)
                                {
                                    standaloneWordAfter = false;
                                    standaloneWordBefore = false;

                                    #region a letter before
                                    if (startIndex != 0)
                                    {
                                        Char letterBefore = element[startIndex - 1];
                                        if (!Char.IsLetterOrDigit(letterBefore))
                                        {
                                            standaloneWordBefore = true;
                                        }
                                        else
                                        {
                                            if (!Char.IsLetterOrDigit(element[startIndex]))
                                            {
                                                standaloneWordBefore = true;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        standaloneWordBefore = true;
                                    }
                                    #endregion

                                    #region a letter after
                                    if (startIndex + term.Length != element.Length)
                                    {
                                        Char letterAfter = element[startIndex + term.Length];
                                        if (!Char.IsLetterOrDigit(letterAfter))
                                        {
                                            standaloneWordAfter = true;
                                        }
                                        else
                                        {
                                            if (!Char.IsLetterOrDigit(element[startIndex + term.Length - 1]))
                                            {
                                                standaloneWordAfter = true;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        standaloneWordAfter = true;
                                    }
                                    #endregion

                                    if (standaloneWordBefore && standaloneWordAfter)
                                    {
                                        indexes.Add(startIndex);
                                    }
                                    startIndex++;
                                }
                            }
                            return indexes;
                        }
                    }
                }

                namespace WordAsPart
                {
                    static class AdvancedSearchEngine
                    {
                        internal static List<int> Contains(string element, string term)
                        {
                            string searchCriteria = term;
                            int startIndex = Const.Ok;
                            List<int> indexes = new List<int>();
                            if (!String.IsNullOrEmpty(element))
                            {
                                while ((startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)) != Const.NotFound)
                                {
                                    indexes.Add(startIndex++);
                                }
                            }
                            return indexes;
                        }
                    }
                }
            }
        }        
    }
}

