using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;
using System.Globalization;

namespace AddinExpress
{
    namespace AdvancedSearch
    {
        public partial class AdvancedSearchSettings : Form
        {
            private GlobalSettings settings;
            internal AdvancedSearchSettings(GlobalSettings settings)
            {
                this.settings = settings;
                InitializeComponent();
                InitializeSettings();
                InitializeGUI();
            }

            private void InitializeGUI()
            {
                char c = '\0';
                CultureInfo adxCulture = CultureInfo.InvariantCulture;
                this.comboBoxShortcuts.BeginUpdate();
                this.comboBoxShortcuts.Items.Add("(none)");
                for (int i = 0x30; i <= 90; i++)
                {
                    c = Convert.ToChar(i);
                    if (char.IsLetter(c))
                    {
                        this.comboBoxShortcuts.Items.Add("Ctrl+" + Char.ToUpper(c, adxCulture));
                        this.comboBoxShortcuts.Items.Add("Shift+" + Char.ToUpper(c, adxCulture));
                        this.comboBoxShortcuts.Items.Add("Ctrl+Shift+" + Char.ToUpper(c, adxCulture));
                    }
                    else if (char.IsDigit(c))
                    {
                        this.comboBoxShortcuts.Items.Add("Ctrl+D" + Char.ToUpper(c, adxCulture));
                        this.comboBoxShortcuts.Items.Add("Shift+D" + Char.ToUpper(c, adxCulture));
                        this.comboBoxShortcuts.Items.Add("Ctrl+Shift+D" + Char.ToUpper(c, adxCulture));
                    }
                }
                for (int j = 1; j <= 12; j++)
                {
                    this.comboBoxShortcuts.Items.Add("Ctrl+F" + j.ToString());
                    this.comboBoxShortcuts.Items.Add("Shift+F" + j.ToString());
                    this.comboBoxShortcuts.Items.Add("Ctrl+Shift+F" + j.ToString());
                }
                this.AddAdditionalShortcuts("Ctrl");
                this.AddAdditionalShortcuts("Shift");
                this.AddAdditionalShortcuts("Ctrl+Shift");
                this.comboBoxShortcuts.SelectedItem = settings.Shortcut;
                this.comboBoxShortcuts.EndUpdate();
                labelVersion.Text = "Advanced search for \n\rInternet Explorer\n\rver." + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
            private void AddAdditionalShortcuts(String extension)
            {
                this.comboBoxShortcuts.Items.Add(extension + "+Insert");
                this.comboBoxShortcuts.Items.Add(extension + "+Delete");
                this.comboBoxShortcuts.Items.Add(extension + "+Back");
                this.comboBoxShortcuts.Items.Add(extension + "+Divide");
                this.comboBoxShortcuts.Items.Add(extension + "+Multiply");
                this.comboBoxShortcuts.Items.Add(extension + "+Subtract");
                this.comboBoxShortcuts.Items.Add(extension + "+Clear");
                this.comboBoxShortcuts.Items.Add(extension + "+Add");
                this.comboBoxShortcuts.Items.Add(extension + "+Escape");
                this.comboBoxShortcuts.Items.Add(extension + "+Space");
                this.comboBoxShortcuts.Items.Add(extension + "+LWin");
                this.comboBoxShortcuts.Items.Add(extension + "+RWin");
                this.comboBoxShortcuts.Items.Add(extension + "+Apps");
                this.comboBoxShortcuts.Items.Add(extension + "+NumLock");
                this.comboBoxShortcuts.Items.Add(extension + "+Up");
                this.comboBoxShortcuts.Items.Add(extension + "+Down");
                this.comboBoxShortcuts.Items.Add(extension + "+Right");
                this.comboBoxShortcuts.Items.Add(extension + "+Left");
                this.comboBoxShortcuts.Items.Add(extension + "+Home");
                this.comboBoxShortcuts.Items.Add(extension + "+End");
                this.comboBoxShortcuts.Items.Add(extension + "+Pause");
                this.comboBoxShortcuts.Items.Add(extension + "+Cancel");
                this.comboBoxShortcuts.Items.Add(extension + "+Prior");
                this.comboBoxShortcuts.Items.Add(extension + "+Next");
                this.comboBoxShortcuts.Items.Add(extension + "+Oemtilde");
                this.comboBoxShortcuts.Items.Add(extension + "+OemMinus");
                this.comboBoxShortcuts.Items.Add(extension + "+OemPlus");
                this.comboBoxShortcuts.Items.Add(extension + "+Oemcomma");
                this.comboBoxShortcuts.Items.Add(extension + "+OemPeriod");
                this.comboBoxShortcuts.Items.Add(extension + "+OemQuestion");
                this.comboBoxShortcuts.Items.Add(extension + "+OemPipe");
                this.comboBoxShortcuts.Items.Add(extension + "+Enter");
                this.comboBoxShortcuts.Items.Add(extension + "+OemSemicolon");
                this.comboBoxShortcuts.Items.Add(extension + "+OemQuotes");

            }
            private void InitializeSettings()
            {              
                this.panelBackColor.BackColor = settings.BackgroundColor;
                this.panelTextColor.BackColor = settings.TextColor;
                this.checkBold.Checked = settings.IsBold;
                this.checkItalic.Checked = settings.IsItalic;
                this.checkNonVisibleHighlight.Checked = settings.IsHightlightNonVisible;
                this.lblTest.ForeColor = settings.TextColor;
                this.lblTest.BackColor = settings.BackgroundColor;
                this.comboBoxShortcuts.SelectedItem = settings.Shortcut;
            }

            private void btnChooseBackColor_Click(object sender, EventArgs e)
            {
                if (colorBackDialog.ShowDialog() == DialogResult.OK)
                {
                    panelBackColor.BackColor = colorBackDialog.Color;
                    lblTest.BackColor = colorBackDialog.Color;
                }
            }
            private void btnChooseTextColor_Click(object sender, EventArgs e)
            {
                ColorDialog dialog = new ColorDialog();
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    panelTextColor.BackColor = dialog.Color;
                    lblTest.ForeColor = dialog.Color;
                }
            }
            private void checkBold_CheckedChanged(object sender, EventArgs e)
            {
                if (checkBold.Checked)
                {
                    lblTest.Font = new Font(lblTest.Font, lblTest.Font.Style | FontStyle.Bold);
                }
                else
                {
                    lblTest.Font = new Font(lblTest.Font, lblTest.Font.Style ^ FontStyle.Bold);
                }
            }
            private void checkItalic_CheckedChanged(object sender, EventArgs e)
            {
                if (checkItalic.Checked)
                {
                    lblTest.Font = new Font(lblTest.Font, lblTest.Font.Style | FontStyle.Italic);
                }
                else
                {
                    lblTest.Font = new Font(lblTest.Font, lblTest.Font.Style ^ FontStyle.Italic);
                }
            }           
            
            public void SynchronizeSettings()
            {
                settings.BackgroundColor = this.panelBackColor.BackColor;
                settings.TextColor = this.panelTextColor.BackColor;
                settings.IsBold = this.checkBold.Checked;
                settings.IsItalic = this.checkItalic.Checked;
                settings.IsHightlightNonVisible = this.checkNonVisibleHighlight.Checked;
                settings.Shortcut = comboBoxShortcuts.SelectedItem as String;
            }                       
            public void DiscardSettings()
            {
                checkBold.Checked = settings.IsBold;
                checkItalic.Checked = settings.IsItalic;
                checkNonVisibleHighlight.Checked = settings.IsHightlightNonVisible;                
                panelBackColor.BackColor = settings.BackgroundColor;
                panelTextColor.BackColor = settings.TextColor;
                panelTest.BackColor = settings.BackgroundColor;
                lblTest.BackColor = settings.BackgroundColor;
                lblTest.ForeColor = settings.TextColor;
                comboBoxShortcuts.SelectedItem = settings.Shortcut;
            }
        }
    }
}