using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.InteropServices;

namespace AddinExpress
{
    namespace AdvancedSearch
    {
        [ComVisible(false)]
        public class GlobalData
        {
            private static GlobalData instance = null;
            private static readonly string path;
            private GlobalSettings settings;

            public static GlobalData Instance
            {
                get
                {
                    if (instance == null)
                        instance = new GlobalData();
                    return instance;
                }
            }
            static GlobalData()
            {
                string remainder = "AdvancedSearch" + Path.DirectorySeparatorChar + System.Environment.UserName + Path.DirectorySeparatorChar + "Settings_cs.dat";
                path = Path.Combine(System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), remainder);
            }
            public GlobalData()
            {
                if (File.Exists(path))
                {
                    using (FileStream fStream = new FileStream(path, FileMode.Open, FileAccess.Read))
                    {
                        BinaryFormatter bFormatter = new BinaryFormatter();
                        settings = (GlobalSettings)bFormatter.Deserialize(fStream);
                    }
                }
                else
                {
                    settings = new GlobalSettings(SystemColors.ButtonFace, SystemColors.ControlText, false, false, false, "Ctrl+F");
                }
            }

            public void SaveSettings()
            {
                if (!Directory.Exists(Path.GetDirectoryName(path)))
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(path));
                }
                using (FileStream fStream = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write))
                {
                    BinaryFormatter bFormatter = new BinaryFormatter();
                    bFormatter.Serialize(fStream, settings);
                }
            }

            public GlobalSettings Settings
            {
                get
                {
                    return this.settings;
                }
            }
        }
    }
}

