Imports System.Collections.Generic
Imports System.Text
Imports Microsoft.Win32
Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports IE = SHDocVw
Imports System.Diagnostics
Imports mshtml

Namespace AdvancedSearch

    <ComVisible(False)> _
    Public Structure ModuleDescriptor
        Public HTMLDoc As mshtml.HTMLDocument
        Public TabWnd As IntPtr
        Public NativeWnd As IntPtr
        Public MainWnd As IntPtr
        Public ThreadId As Integer
        Public Form As HTMLForm
    End Structure

    <ComVisible(False)> _
    Public Structure SearchDescriptor
        Public CheckCase As Boolean
        Public CheckHidden As Boolean
        Public CheckTabs As Boolean
        Public CheckWholeWord As Boolean
        Public CheckWindows As Boolean
        Public Term As String
        Public Flags As SearchFlags
        Public SearchModuleIndex As Integer
        Public CurrentModuleIndex As Integer
        Public Modules As List(Of ModuleDescriptor)
        Public ResultNode As TreeNode
    End Structure

    <ComVisible(False)> _
    <Serializable()> _
    Public Class HtmlInfoTag
        Public Sub New(ByVal line As Integer, ByVal position As Integer)
            Me._line = line
            Me._position = position
        End Sub

        Private _line As Integer
        Private _position As Integer

        Public ReadOnly Property Line() As Integer
            Get
                Return Me._line
            End Get
        End Property
        Public ReadOnly Property Position() As Integer
            Get
                Return Me._position
            End Get
        End Property
    End Class

    <ComVisible(False)> _
    <Serializable()> _
    Public Class WindowInfoTag
        Public Sub New(ByVal moduleDesc As ModuleDescriptor, ByVal term As String, ByVal flags As SearchFlags)
            Me._moduleDesc = moduleDesc
            Me._flags = flags
            Me._term = term
        End Sub

        Private _moduleDesc As ModuleDescriptor
        Private _flags As SearchFlags
        Private _term As String

        Public ReadOnly Property NativeWindowHandle() As IntPtr
            Get
                Return _moduleDesc.NativeWnd
            End Get
        End Property
        Public ReadOnly Property HWND() As IntPtr
            Get
                Return Me._moduleDesc.MainWnd
            End Get
        End Property
        Public ReadOnly Property ThreadID() As Integer
            Get
                Return Me._moduleDesc.ThreadId
            End Get
        End Property

        Public ReadOnly Property Term() As String
            Get
                Return Me._term
            End Get
        End Property

        Public ReadOnly Property Flags() As SearchFlags
            Get
                Return Me._flags
            End Get
        End Property
    End Class

    <ComVisible(False)> _
    <Flags()> _
    <Serializable()> _
    Public Enum SearchFlags
        ieMatchPartialWords = 0
        ieMatchBackwards = 1
        ieMatchWholeWordsOnly = 2
        ieMatchCase = 4
    End Enum

    <ComVisible(False)> _
    Public Enum SearchProgress
        None = -1
        Start = 0
        Initializing = 10
        Searching = 50
        AddingNodes = 80
        Done = 100
    End Enum

    <ComVisible(False)> _
    Public Class Constant
        'auxiliary
        Public Shared ReadOnly Ok As Integer = 0
        Public Shared ReadOnly Found As Integer = 0
        Public Shared ReadOnly NotFound As Integer = -1
        Public Shared ReadOnly None As Integer = NotFound
        'Internet Explorer
        Public Shared ReadOnly IE6 As Integer = 6
        Public Shared ReadOnly IE7 As Integer = 7
        'window messages
        Public Shared ReadOnly WM_USER As Integer = 1024
        Public Shared ReadOnly WM_SETFOCUS As Integer = 7
        Public Shared ReadOnly WM_ACTIVATE As Integer = 6
        Public Shared ReadOnly WM_SHOWWINDOW As Integer = 18
        'custom
        Public Shared ReadOnly WM_SELECT_TAB As Integer = WM_USER + 1000
        Public Shared ReadOnly WM_ITEM_CLOSED As Integer = WM_USER + 10000
        Public Shared ReadOnly WM_SHOW_HTML_FORM As Integer = WM_USER + 1111
        Public Shared ReadOnly WM_CLOSE_FORMS As Integer = WM_USER + 1112
    End Class

    Namespace VisibleContent
        Namespace CaseSensitive
            Namespace WholeWord
                Class AdvancedSearchEngine
                    Friend Shared Function Contains(ByVal element As String, ByVal _term As String) As List(Of Integer)
                        Dim searchCriteria As String = _term
                        Dim startIndex As Integer = Constant.Ok
                        Dim indexes As List(Of Integer) = New List(Of Integer)()
                        Dim standaloneWordAfter As Boolean = False
                        Dim standaloneWordBefore As Boolean = False
                        If (Not String.IsNullOrEmpty(element)) Then
                            startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)
                            While (Not (startIndex = Constant.NotFound))
                                standaloneWordAfter = False
                                standaloneWordBefore = False
                                If Not (startIndex = 0) Then
                                    Dim letterBefore As Char = element(startIndex - 1)
                                    If (Not Char.IsLetterOrDigit(letterBefore)) Then
                                        standaloneWordBefore = True
                                    Else
                                        If (Not Char.IsLetterOrDigit(element(startIndex))) Then
                                            standaloneWordBefore = True
                                        End If
                                    End If
                                Else
                                    standaloneWordBefore = True
                                End If

                                If Not ((startIndex + _term.Length) = element.Length) Then

                                    Dim letterAfter As Char = element(startIndex + _term.Length)
                                    If (Not Char.IsLetterOrDigit(letterAfter)) Then
                                        standaloneWordAfter = True
                                    Else
                                        If (Not Char.IsLetterOrDigit(element(startIndex + _term.Length - 1))) Then

                                            standaloneWordAfter = True
                                        End If
                                    End If
                                Else
                                    standaloneWordAfter = True
                                End If
                                If (standaloneWordBefore And standaloneWordAfter) Then
                                    indexes.Add(startIndex)
                                End If
                                startIndex += 1
                                startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)
                            End While
                        End If
                        Return indexes
                    End Function
                End Class
            End Namespace

            Namespace WordAsPart
                Class AdvancedSearchEngine
                    Friend Shared Function Contains(ByVal element As String, ByVal _term As String) As List(Of Integer)
                        Dim searchCriteria As String = _term
                        Dim startIndex As Integer = Constant.Ok
                        Dim indexes As List(Of Integer) = New List(Of Integer)()
                        If (Not String.IsNullOrEmpty(element)) Then
                            startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)
                            While (Not (startIndex = Constant.NotFound))
                                indexes.Add(startIndex)
                                startIndex += 1
                                startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.Ordinal)
                            End While
                        End If
                        Return indexes
                    End Function
                End Class
            End Namespace
        End Namespace
        Namespace CaseInsensetive
            Namespace WholeWord
                Class AdvancedSearchEngine
                    Friend Shared Function Contains(ByVal element As String, ByVal _term As String) As List(Of Integer)
                        Dim searchCriteria As String = _term
                        Dim startIndex As Integer = Constant.Ok
                        Dim indexes As List(Of Integer) = New List(Of Integer)()
                        Dim standaloneWordAfter As Boolean = False
                        Dim standaloneWordBefore As Boolean = False
                        If (Not String.IsNullOrEmpty(element)) Then
                            startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)
                            While (Not (startIndex = Constant.NotFound))
                                standaloneWordAfter = False
                                standaloneWordBefore = False
                                If Not (startIndex = 0) Then
                                    Dim letterBefore As Char = element(startIndex - 1)
                                    If (Not Char.IsLetterOrDigit(letterBefore)) Then
                                        standaloneWordBefore = True
                                    Else
                                        If (Not Char.IsLetterOrDigit(element(startIndex))) Then
                                            standaloneWordBefore = True
                                        End If
                                    End If
                                Else
                                    standaloneWordBefore = True
                                End If
                                If Not (startIndex + _term.Length = element.Length) Then
                                    Dim letterAfter As Char = element(startIndex + _term.Length)
                                    If (Not Char.IsLetterOrDigit(letterAfter)) Then
                                        standaloneWordAfter = True
                                    Else
                                        If (Not Char.IsLetterOrDigit(element(startIndex + _term.Length - 1))) Then
                                            standaloneWordAfter = True
                                        End If
                                    End If
                                Else
                                    standaloneWordAfter = True
                                End If
                                If (standaloneWordBefore And standaloneWordAfter) Then
                                    indexes.Add(startIndex)
                                End If
                                startIndex += 1
                                startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)
                            End While
                        End If
                        Return indexes
                    End Function
                End Class
            End Namespace

            Namespace WordAsPart
                Class AdvancedSearchEngine
                    Friend Shared Function Contains(ByVal element As String, ByVal _term As String) As List(Of Integer)
                        Dim searchCriteria As String = _term
                        Dim startIndex As Integer = Constant.Ok
                        Dim indexes As List(Of Integer) = New List(Of Integer)()
                        If (Not String.IsNullOrEmpty(element)) Then
                            startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)
                            While (Not (startIndex = Constant.NotFound))
                                indexes.Add(startIndex)
                                startIndex += 1
                                startIndex = element.IndexOf(searchCriteria, startIndex, StringComparison.OrdinalIgnoreCase)
                            End While
                        End If
                        Return indexes
                    End Function
                End Class
            End Namespace

        End Namespace
    End Namespace
End Namespace
